/*
 Basic ESP8266 MQTT example

 This sketch demonstrates the capabilities of the pubsub library in combination
 with the ESP8266 board/library.

 It connects to an MQTT server then:
  - publishes "hello world" to the topic "outTopic" every two seconds
  - subscribes to the topic "inTopic", printing out any messages
    it receives. NB - it assumes the received payloads are strings not binary
  - If the first character of the topic "inTopic" is an 1, switch ON the ESP Led,
    else switch it off

 It will reconnect to the server if the connection is lost using a blocking
 reconnect function. See the 'mqtt_reconnect_nonblocking' example for how to
 achieve the same result without blocking the main loop.

 To install the ESP8266 board, (using Arduino 1.6.4+):
  - Add the following 3rd party board manager under "File -> Preferences -> Additional Boards Manager URLs":
       http://arduino.esp8266.com/stable/package_esp8266com_index.json
  - Open the "Tools -> Board -> Board Manager" and click install for the ESP8266"
  - Select your ESP8266 in "Tools -> Board"

*/

#include <ESP8266WiFi.h>
#include <PubSubClient.h>

/*
    Update these with values suitable for your network.
    Пропиши свою сеть
*/

const char* ssid = "........";
const char* password = "........";

/*
 * Если брокер локальный
 */
//const char* mqtt_server = "192.168.1.1";

/*
 * Если брокер в облаке. Имя брокера
 */
const char* mqtt_server = "wqtt.ru";

/*
 * Юзер и пароль для брокера
 * Если их нет , закомментировать и выбрать строку 195
 */
 
const char *mqtt_user = "IotManager:guest"; // Пользователь брокера
const char *mqtt_pass = "guest"; // Пароль от брокера




WiFiClient espClient;
PubSubClient client(espClient);

#include <Wire.h>
#include <LiquidCrystal_I2C.h>

long lastMsg = 0;
char msg[50];
int value = 0;

LiquidCrystal_I2C lcd(0x27,20,4);//set the LCD address to 0x27 for a 16 chars and 2 line display или 20 на 4

void setup_wifi() {

  delay(10);
  // We start by connecting to a WiFi network
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);

  WiFi.begin(ssid, password);

  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }

  randomSeed(micros());

  Serial.println("");
  Serial.println("WiFi connected");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());
}

void callback(char* topic, byte* payload, unsigned int length) {
  Serial.print("Message arrived [");
  Serial.print(topic);
  Serial.print("] ");
   
   String value;
  for (int i = 0; i < length; i++) {

       value += (char)payload[i];

       Serial.print((char)payload[i]);
     

//============= Топики которые выводятся на экран ===============================

/*
 * Подписываемся на топик для ble датчиков . Заменить на свои 
 */

   if (strcmp(topic, "/IotManager/28xxxxx-145xxxxxx/LYWSD02321C22_Temperature/control") == 0) { // "/Ваш_префикс/ID_8266/Ваш_топик/control"
            
    lcd.setCursor(10, 1);              // Позиция вывода на экран. Номер в строке , номер строки
 
   lcd.print(value);
   
/*
 * Любой топик из IoTmanager
 */

   }else if (strcmp(topic, "/IotManager/158xxxxx-145xxxx/tmp773/event") == 0) {
            
    lcd.setCursor(10, 1);
 
   lcd.print(value);

   /*
    * Топик из MQTT брокера проекта MySensors
    */
  }else if (strcmp(topic, "mygateway2-out/48/11/1/0/25") == 0) {
 
       lcd.setCursor(10, 2);
 
   lcd.print(value);


 }else if (strcmp(topic, "/IotManager/158xxxxx-1458xxxx/tmp110/event") == 0) {

       lcd.setCursor(5, 1);
       
       lcd.print(value); 
  }else if (strcmp(topic, "/IotManager/286xxxx-145xxxxx/LYWSD02321C22_Humidity/control") == 0) {

       lcd.setCursor(5, 2);
       lcd.print(value);

  }else if (strcmp(topic, "/IotManager/286xxxx-1458xxxx/LYWSD02321C22_Battery/control") == 0) {

       lcd.setCursor(5, 3);
       
       lcd.print(value);
   
   }else if (strcmp(topic, "/IotManager/286xxxx-1458xxxxx/ATC5AA4AA_Battery/control") == 0) {
  

       lcd.setCursor(10, 3 );
 
   lcd.print(value);
   }else if (strcmp(topic, "/IotManager/286xxxxx-145xxxxx/LYWSD02321C22_last/control") == 0) {
  

       lcd.setCursor(0, 0 );
 
   lcd.print(value);
   }else if (strcmp(topic, "/IotManager/163xxxx-145xxxxx/tmp170/event") == 0) {
  

       lcd.setCursor(15, 3 );
 
   lcd.print(value);
   }
  }
  Serial.println();

  }

void reconnect() {
  // Loop until we're reconnected
  while (!client.connected()) {                                         
    Serial.print("Attempting MQTT connection...");                  
    // Create a random client ID
    String clientId = "ESP8266Client-";
    clientId += String(random(0xffff), HEX);
    // Attempt to connect

    /*
     * Если есть авторизация . Если нет , закомментировать и разкомментировать следующую строчку
     */
   if (client.connect(clientId.c_str(), mqtt_user, mqtt_pass)) {                            
    //      if (client.connect(clientId.c_str())) {                             
                                                                       
                                                                   
        
      Serial.println("connected");


// ================  Подписка на нужные топики =================================
      

     uint16_t packetIdSub =  client.subscribe("/IotManager/28xxxx-145xxxxx/LYWSD02321C22_Temperature/control"); // Данные с BLE датчиков
     uint16_t packetIdSub1 = client.subscribe("/IotManager/28xxxx-145xxxxx/LYWSD02321C22_Humidity/control");
     uint16_t packetIdSub2 = client.subscribe("/IotManager/28xxxx-145xxxxx/LYWSD02321C22_Battery/control");
     uint16_t packetIdSub3 = client.subscribe("/IotManager/28xxxx-145xxxxx/ATC5AA4AA_Temperature/control");
     uint16_t packetIdSub4 = client.subscribe("/IotManager/28xxxx-145xxxxx/ATC5AA4AA_Humidity/control");
     uint16_t packetIdSub5 = client.subscribe("/IotManager/28xxxx-145xxxxx/ATC5AA4AA_Battery/control");
     uint16_t packetIdSub6 = client.subscribe("/IotManager/28xxxx-145xxxxx/LYWSD02321C22_last/control");
     uint16_t packetIdSub7 = client.subscribe("/IotManager/28xxxx-145xxxxx/upt372/status"); // Это не работает нормально 
     uint16_t packetIdSub8 = client.subscribe("/IotManager/15xxxx-145xxxxx/tmp110/event");  // Температура Далласа1 на первой ЕСП8266
     uint16_t packetIdSub9 = client.subscribe("/IotManager/16xxxx-145xxxxx/tmp744/event");  // Температура Далласа2 на первой ЕСП8266
     uint16_t packetIdSub10 = client.subscribe("/IotManager/16xxxx-145xxxxx/upt904/event");

     uint16_t packetIdSub11 = client.subscribe("/IotManager/28xxxxx-145xxxx/MJ_HT_V13B4711_last/control");
     uint16_t packetIdSub12 = client.subscribe("/IotManager/28xxxxx-145xxxx/MJ_HT_V13B4711_Temperature/control");
     uint16_t packetIdSub13 = client.subscribe("/IotManager/28xxxxx-145xxxx/MJ_HT_V13B4711_Humidity/control");
     uint16_t packetIdSub14 = client.subscribe("/IotManager/28xxxxx-145xxxx/MJ_HT_V13B4711_Battery/control");
     uint16_t packetIdSub15 = client.subscribe("/IotManager/28xxxxx-145xxxx/MJ_HT_V13B4711_RSSI/control");
     uint16_t packetIdSub16 = client.subscribe("/IotManager/28xxxxx-145xxxx/MJ_HT_V13B4711_DewPoint/control");

     uint16_t packetIdSub17 = client.subscribe("/IotManager/158xxxx-145xxxx/tmp773/event");
     uint16_t packetIdSub18 = client.subscribe("mygateway2-out/48/11/1/0/25");

  Serial.print(packetIdSub);
  Serial.print(packetIdSub1);
  Serial.print(packetIdSub2);
  Serial.print(packetIdSub3);
  Serial.print(packetIdSub4);
  Serial.print(packetIdSub5);
  Serial.print(packetIdSub6);
  Serial.print(packetIdSub7);
  Serial.print(packetIdSub8);
  Serial.print(packetIdSub9);
  Serial.print(packetIdSub10);
  Serial.print(packetIdSub11);
  Serial.print(packetIdSub12);
  Serial.print(packetIdSub13);
  Serial.print(packetIdSub14);
  Serial.print(packetIdSub15);
  Serial.print(packetIdSub16);
  Serial.print(packetIdSub17);
  Serial.println(packetIdSub18);


    } else {
      Serial.print("failed, rc=");
      Serial.print(client.state());
      Serial.println(" try again in 5 seconds");
      // Wait 5 seconds before retrying
      delay(5000);
    }
  }
}

void setup() {
   lcd.begin();
  Serial.begin(115200);
 setup_wifi();
 // client.setServer(mqtt_server, 18883);
 client.setServer(mqtt_server, 1883);
  client.setCallback(callback);
/*
 * Выводится один раз и висит постоянно
 */

    lcd.setCursor(5, 0);
  
    lcd.print("IoTmanager");  //  " Что угодно"

        lcd.setCursor(0, 1);
   
    lcd.print("Temp");  //  " Температура"

            lcd.setCursor(0, 2);
   
    lcd.print("Humi");  //  " Влажность"

lcd.setCursor(0, 3);
   // lcd.write(0);
    lcd.print("Batt");  //  " Батарейка"
}

void loop() {

  if (!client.connected()) {
    reconnect();
  }
  client.loop();



}
